/**
 * @fileoverview Environment globals for p5.js
 * @author Mark Sherman
 */
"use strict";

//------------------------------------------------------------------------------
// Plugin Definition
//------------------------------------------------------------------------------
// Functions that are implicitly called by p5 and should not trigger the no-unused-vars rule
var implicitCallFunctionsList = [
  "preload",
  "setup",
  "draw",
  "windowResized",
  "deviceMoved",
  "deviceTurned",
  "deviceShaken",
  "keyTyped",
  "keyPressed",
  "keyReleased",
  "mouseMoved",
  "mouseDragged",
  "mousePressed",
  "mouseReleased",
  "mouseClicked",
  "doubleClicked",
  "mouseWheel",
  "touchStarted",
  "touchMoved",
  "touchEnded",
];
// Process the list into a regexp for the eslint rule
var implicitCallFunctions = implicitCallFunctionsList.join('|');

// Collect the names of the read-only globals by section, then combine them
let p5 = {};
p5.base = [
  "remove",
  "canvas",
  "width",
  "height",
  "drawingContext",
  "frameCount",
  "pmouseX",
  "pmouseY",
  "mouseX",
  "mouseY",
  "pwinMouseX",
  "pwinMouseY",
  "ptouchX",
  "ptouchY",
  "touchX",
  "touchY",
  "registerPreloadMethod",
  "registerMethod",
  "alpha",
  "blue",
  "brightness",
  "color",
  "green",
  "hue",
  "lerpColor",
  "lightness",
  "red",
  "saturation",
  "background",
  "clear",
  "colorMode",
  "fill",
  "noFill",
  "noStroke",
  "stroke",
  "float",
  "int",
  "str",
  "boolean",
  "byte",
  "char",
  "unchar",
  "hex",
  "unhex",
  "append",
  "arrayCopy",
  "concat",
  "reverse",
  "shorten",
  "shuffle",
  "sort",
  "splice",
  "subset",
  "join",
  "match",
  "matchAll",
  "nf",
  "nfc",
  "nfp",
  "nfs",
  "split",
  "splitTokens",
  "trim",
  "print",
  "println",
  "deltaTime",
  "focused",
  "cursor",
  "frameRate",
  "getFrameRate",
  "setFrameRate",
  "noCursor",
  "displayWidth",
  "displayHeight",
  "windowWidth",
  "windowHeight",
  "fullscreen",
  "pixelDensity",
  "displayDensity",
  "getURL",
  "getURLPath",
  "getURLParams",
  "createImage",
  "saveCanvas",
  "saveFrames",
  "loadImage",
  "image",
  "tint",
  "noTint",
  "imageMode",
  "pixels",
  "blend",
  "copy",
  "filter",
  "get",
  "loadPixels",
  "set",
  "updatePixels",
  "loadFont",
  "createInput",
  "createReader",
  "loadBytes",
  "loadJSON",
  "loadStrings",
  "loadTable",
  "loadXML",
  "parseXML",
  "selectFolder",
  "selectInput",
  "httpGet",
  "httpPost",
  "httpDo",
  "beginRaw",
  "beginRecord",
  "createOutput",
  "createWriter",
  "endRaw",
  "endRecord",
  "saveBytes",
  "save",
  "saveJSON",
  "saveJSONObject",
  "saveJSONArray",
  "saveStream",
  "saveStrings",
  "saveXML",
  "selectOutput",
  "saveTable",
  "writeFile",
  "downloadFile",
  "isKeyPressed",
  "keyIsPressed",
  "key",
  "keyCode",
  "keyIsDown",
  "deviceOrientation",
  "accelerationX",
  "accelerationY",
  "accelerationZ",
  "pAccelerationX",
  "pAccelerationY",
  "pAccelerationZ",
  "rotationX",
  "rotationY",
  "rotationZ",
  "pRotationX",
  "pRotationY",
  "pRotationZ",
  "turnAxis",
  "setMoveThreshold",
  "setShakeThreshold",
  "winMouseX",
  "winMouseY",
  "mouseButton",
  "mouseIsPressed",
  "isMousePressed",
  "day",
  "hour",
  "minute",
  "millis",
  "month",
  "second",
  "year",
  "touches",
  "touchIsDown",
  "createVector",
  "abs",
  "ceil",
  "constrain",
  "dist",
  "exp",
  "floor",
  "lerp",
  "log",
  "mag",
  "map",
  "max",
  "min",
  "norm",
  "pow",
  "round",
  "sq",
  "sqrt",
  "randomSeed",
  "random",
  "randomGaussian",
  "noise",
  "noiseDetail",
  "noiseSeed",
  "acos",
  "asin",
  "atan",
  "atan2",
  "cos",
  "sin",
  "tan",
  "degrees",
  "radians",
  "angleMode",
  "createCanvas",
  "resizeCanvas",
  "noCanvas",
  "createGraphics",
  "blendMode",
  "arc",
  "ellipse",
  "circle",
  "line",
  "point",
  "quad",
  "rect",
  "triangle",
  "ellipseMode",
  "noSmooth",
  "rectMode",
  "smooth",
  "strokeCap",
  "strokeJoin",
  "strokeWeight",
  "bezier",
  "bezierDetail",
  "bezierPoint",
  "bezierTangent",
  "curve",
  "curveDetail",
  "curveTightness",
  "curvePoint",
  "curveTangent",
  "beginContour",
  "beginShape",
  "bezierVertex",
  "curveVertex",
  "endContour",
  "endShape",
  "quadraticVertex",
  "vertex",
  "exit",
  "noLoop",
  "loop",
  "push",
  "pop",
  "pushStyle",
  "popStyle",
  "redraw",
  "size",
  "applyMatrix",
  "popMatrix",
  "printMatrix",
  "pushMatrix",
  "resetMatrix",
  "rotate",
  "rotateX",
  "rotateY",
  "rotateZ",
  "scale",
  "shearX",
  "shearY",
  "translate",
  "textAlign",
  "textLeading",
  "textSize",
  "textStyle",
  "textWidth",
  "textAscent",
  "textDescent",
  "text",
  "textFont",
  "plane",
  "sphere",
  "ellipsoid",
  "cylinder",
  "cone",
  "torus",
  "box",
  "normalMaterial",
  "texture",
  "basicMaterial",
  "ambientMaterial",
  "specularMaterial",
  "ambientLight",
  "directionalLight",
  "pointLight",
  "camera",
  "perspective",
  "ortho",
  "orbitControl",
  "P2D",
  "WEBGL",
  "ARROW",
  "CROSS",
  "HAND",
  "MOVE",
  "TEXT",
  "WAIT",
  "HALF_PI",
  "PI",
  "QUARTER_PI",
  "TAU",
  "TWO_PI",
  "DEGREES",
  "RADIANS",
  "CORNER",
  "CORNERS",
  "RADIUS",
  "RIGHT",
  "LEFT",
  "CENTER",
  "TOP",
  "BOTTOM",
  "BASELINE",
  "POINTS",
  "LINES",
  "TRIANGLES",
  "TRIANGLE_FAN",
  "TRIANGLE_STRIP",
  "QUADS",
  "QUAD_STRIP",
  "CLOSE",
  "OPEN",
  "CHORD",
  "PIE",
  "PROJECT",
  "SQUARE",
  "ROUND",
  "BEVEL",
  "MITER",
  "RGB",
  "HSB",
  "HSL",
  "AUTO",
  "ALT",
  "BACKSPACE",
  "CONTROL",
  "DELETE",
  "DOWN_ARROW",
  "ENTER",
  "ESCAPE",
  "LEFT_ARROW",
  "OPTION",
  "RETURN",
  "RIGHT_ARROW",
  "SHIFT",
  "TAB",
  "UP_ARROW",
  "BLEND",
  "ADD",
  "DARKEST",
  "LIGHTEST",
  "DIFFERENCE",
  "EXCLUSION",
  "MULTIPLY",
  "SCREEN",
  "REPLACE",
  "OVERLAY",
  "HARD_LIGHT",
  "SOFT_LIGHT",
  "DODGE",
  "BURN",
  "THRESHOLD",
  "GRAY",
  "OPAQUE",
  "INVERT",
  "POSTERIZE",
  "DILATE",
  "ERODE",
  "BLUR",
  "NORMAL",
  "ITALIC",
  "BOLD",
  "LINEAR",
  "QUADRATIC",
  "BEZIER",
  "CURVE",
];
p5.sound = [
  "getAudioContext",
  "isSupported",
  "isFileSupported",
  "getMasterVolume",
  "masterVolume",
  "sampleRate",
  "freqToMidi",
  "midiToFreq",
  "soundFormats",
  "disposeSound",
  "loadSound",
  "createConvolver",
  "setBPM",
  "saveSound",
];
p5.dom = [
  "select",
  "selectAll",
  "removeElements",
  "createDiv",
  "createP",
  "createSpan",
  "createImg",
  "createA",
  "createSlider",
  "createButton",
  "createCheckbox",
  "createSelect",
  "createRadio",
  "createColorPicker",
  "createInput",
  "createFileInput",
  "createVideo",
  "createAudio",
  "VIDEO",
  "AUDIO",
  "createCapture",
  "createElement",
];

let p5Globals = {};
for (const sec in p5) {
  for (const g of p5[sec]) {
    p5Globals[g] = "readonly";
  }
}

module.exports = {
  configs: {
    p5: {
      rules: {
        "no-unused-vars": [
          "warn",
          { varsIgnorePattern: implicitCallFunctions },
        ],
      },
    },
  },
  environments: {
    p5: {
      globals: p5Globals,
    },
  },
};
